clear all; 
close all;
clc;
matpath = '/home/arnold/matlab';
load('plants.mat', 'plants')

figpath   = 'latex/acc2016_finalForm/figs';
fig1_name = 'fto_xqref_assym.eps';
fig1_path = fullfile(figpath, fig1_name);
%%%%%%%%%%%%%%%%%%%%%%%
% DEFINE GLOBAL AND HANDLES
f = @dyn;

TT = 1;

saveOn = 0; % flage to save figure.
%%%%%%%%%%%%%%%%%%%%%%%
% Setup dynamics

% GET LRC PLANT 
sys = plants.Gss;
A = sys.a; B = sys.b; C = sys.c;
A(2,2) = 0;
w = sqrt(-A(2,1));
wd =w;
bo = B(2);
B(2) = bo;
c = bo/w^2;

% define the reference
x1r = -0.4*c;
% form umin umax
umax = 1 - x1r/c;
umin = -1 - x1r/c;

% setup time vector field
dt = .00001;
dt = 1e-5;
t = (0:dt:pi/wd)';



% Confirm analytic version
dx = c/500;
x1_max   = (0:dx:2*umax*c)';
fmax = -w*sqrt(2*umax*c*abs(x1_max) - x1_max.^2);

x1_min   = (2*umin*c:dx:0)';
fmin = w*sqrt(2*umin*c*x1_min - x1_min.^2);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% GENERATE SWITCHING CURVE IN UNSHIFTED COORDS. UMIN/MAX = +-1;, 

t = flipud(t); % integrate backwards
Xf = [x1r 0]';


params.A = A;
params.B = B;
params.u = 1;

[t, x_p]=ode45(@(t,x)f(t,x,params), t, Xf);
params.u = -1;
[t, x_n]=ode45(@(t,x)f(t,x,params), t, Xf);

% enforce monotic
Scurve = [flipud(x_n); x_p];
Scurve = forceMono(Scurve);



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  NOW PRODUCE THE PLOT IN THE ERROR CORRDINATES
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% SETUP FIGURE PROPERTIES
f1=figure(2);

height=1.65/1.618;
width=1.4;
xpos=469;
ypos=546;
scale=350;


axes('FontName','Times New Roman') % Set axis font style
box('on'); % Define box around whole figure
set(gcf,'Position',[xpos ypos scale*width scale*height]) % Set figure format
hold on, grid on

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% PLOT CURVES

p6     = plot(x1_max, fmax, 'r', 'LineWidth', 2);
p7     = plot(x1_min, fmin, 'r', 'LineWidth', 2);

ylim([-11000, 5000]);
ylm = ylim;
xlim([-1.7, 3.]);
xlm = xlim;
%plot FTO approx outside 
plot([xlm(1), x1_min(1)], [0 0], 'r', 'LineWidth',2)
plot([x1_max(end) xlm(2)], [0 0], 'r', 'LineWidth',2)

%%%%%% build the axes
% =============================================================
% *************************************************************
clc
ax=gca;

% xlim([1.2*min(min(xn_s(:, kx1s))),  1.2*max(max(xp_s(:, kx1s)))]);
xlm = xlim;
Nx1   = ceil(xlm(1)/c);
Nx2   = floor(xlm(2)/c);
nxdiff = 1;
xdiff = nxdiff*c;



set(ax, 'XTick', [Nx1*xdiff/nxdiff:xdiff:Nx2*xdiff/nxdiff]);
set(ax, 'XTickLabel', [])


ylm = ylim;

Ny1    = ceil(ylm(1)/((bo)/w));
Ny2    = floor(ylm(2)/((bo)/w));
nydiff = 1;
ydiff  = nydiff*bo/w;

% % ylim([1.2*min(min(xp_s)), 1.4*max([max(xn_s(:,end))])])
set(ax, 'YTick', [Ny1*ydiff/nydiff:ydiff:Ny2*ydiff/nydiff]);
set(ax, 'YTickLabel', [])


yticks=get(ax, 'ytick');
xticks=get(ax, 'xtick');

%==========
% Build strings for custom labels.
j = 1;
for iter = Nx1:nxdiff:Nx2
    if iter ==1 
       my_xticks{j} = sprintf('$c$'); 
    elseif iter ==-1
        my_xticks{j} = sprintf('$-c$') ;
    elseif iter ==0
        my_xticks{j} = sprintf('$0$');
    else
        my_xticks{j} = sprintf('$%d c$', iter);
    end
j = j+1;
end


j = 1;
for iter = Ny1:nydiff:Ny2
    if iter ==1 
       my_yticks{j} = sprintf('$\\frac{b_o}{w}$'); 
    elseif iter ==-1
        my_yticks{j} = sprintf('$\\frac{ -b_o}{w}$') ;
    elseif iter ==0
        my_yticks{j} = sprintf('$0$');
    else
        my_yticks{j} = sprintf('$\\frac{ %d b_o}{w}$', iter);
    end
j = j+1;
end

xlm = xlim;
ylm = ylim;

axs=axis;
HorizontalOffset=4.1*c;
VertOffset=1.5*1000;


xdist = xlm(2) - xlm(1);
ydist = ylm(2) - ylm(1);

yax = xlm(1);
xax = ylm(1);
for i=1:length(my_yticks)
    
    
  text(yax-.01*c, yticks(i),my_yticks(i),'HorizontalAlignment','Right','interpreter', 'latex',...
      'FontSize', 12);

end

for i=1:length(my_xticks)
  s(i) = text(xticks(i), xax(1) - ydist*.01, my_xticks(i),'HorizontalAlignment','Center',...
      'VerticalAlignment', 'Top', 'interpreter', 'latex', 'FontSize',12);   

end

grid on

% END AXES
% =============================================================
% *************************************************************

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  PLOT VERTICAL LINES
% p5     = plot(c*umax, 0, 'xg', c*umin, 0, 'xg'); 
p_holdeq2 = plot([c*umax, c*umax], [ylm], 'b','LineWidth', 2);
p_holdeq2 = plot([c*umin, c*umin], [ylm], 'b','LineWidth', 2);
p4     = plot(Scurve(:,1), Scurve(:,2), '--r', 'LineWidth', 1);
% p_holdeq = plot([c, c], [ylm], 'k', 'LineWidth', 2)
%            plot([-c, -c], [ylm], 'k', 'LineWidth', 2)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  PLOT MEASURES

plot([c*umin, c*umax], [0 0], 'k')
plot([c*umin+.07, c*umin-.07], [350, -350], 'k')
plot([.07, -.07], [350, -350], 'k')           
plot([c*umax+.07, c*umax-.07], [350, -350], 'k')    
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% LABEL AXES
hxlab = xlabel('$x_{1e}$ (error coordinates)', 'interpreter', 'latex', 'FontSize',12);
hylab = ylabel('$x_{2e}$', 'interpreter', 'latex', 'FontSize', 14, 'rot',0);
set(hxlab, 'Units', 'Normalized', 'Position', [.5, -0.05, 0]);
set(hylab, 'Units', 'Normalized', 'Position', [-.1, 0.5, 0]);



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  PLOT TEXT
h_xln  = text(.22,.045, '$c\cdot \bar u^-$', 'HorizontalAlignment','Right','interpreter', 'latex', 'FontSize', 12, 'Units', 'Normalized');
h_xlp  = text(.74, .045, '$c \cdot \bar u^+$', 'HorizontalAlignment','Right','interpreter', 'latex', 'FontSize', 12, 'Units', 'Normalized');
% h_xlp=text(.2,.05, '$u\delta^- c$', 'HorizontalAlignment','Right','interpreter', 'latex', 'FontSize', 12, 'Units', 'Normalized');
% h_xlp=text(.645,.05, '$u\delta^+ c$', 'HorizontalAlignment','Right','interpreter', 'latex', 'FontSize', 12, 'Units', 'Normalized');

h_aplus = text(.50, .72, '$c \bar u^+$', 'interpreter', 'latex', 'FontSize', 12, 'Units', 'Normalized');
h_amin  = text(.27, .72, '$c \bar u^-$', 'interpreter', 'latex', 'FontSize', 12, 'Units', 'Normalized');

% h_aplusis = text(.42, .52, '$a^- = c\bar u^-$', 'interpreter', 'latex', 'FontSize', 12, 'Units', 'Normalized');
% h_aplusis = text(.42, .42, '$a^+ = c\bar u^+$', 'interpreter', 'latex', 'FontSize', 12, 'Units', 'Normalized');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  CREATE LEGEND
% leg2 = legend([p6, p4, p_holdeq2], { '$f_{to, error}$', '$f_{to}$, backwards from $x_r$', 'holdable eqilibrium limits'}, 'interpreter', 'latex');
leg2 = legend([p6, p_holdeq2], { '$f_{to,e}(x_{1e})$', 'holdable eqilibrium limits'}, 'interpreter', 'latex');
leg2.Position =  [0.4212 0.7408 0.4801 0.1801];
set(p4, 'Visible', 'off')
% leg2.Position = [0.48994 0.80792 0.39269 0.098516];
leg2.FontSize = 12;



%%
if saveOn
    hgexport(gcf, fig1_path, ...
        hgexport('factorystyle'), 'Format', 'epsc')
end



