% Setup some workspace things
% close all,
clear all
close all
clf, clc;


% Figure Path
fig1_path   = 'latex/acc2016_finalForm/figs/udot.eps';

load( 'plants.mat', 'plants')
load('CS.mat', 'CS');
F1         = figure(1);
F1.Position = [-651 570 560 420];
hold on, grid on;



sys = plants.Gss;
A = sys.a;
B = sys.b;

f = @dyn;


% Setup dynamics
w = sqrt(-A(2,1));
z = 0.5*0;
A(2,2) = 0;


params.A = A;
params.B = B;


bo = B(2);
c = bo/w^2;
wd = w*sqrt(1-z^2);

% what tolerance??
x1Tol = c/100;
x2Tol = (bo/w)/100;

sys = ss(A, B, [1 0], 0);
% setup time vector field
dt = (pi/wd)/1000;
t = (0:dt:pi/wd)';
t = flipud(t); % integrate backwards

% HOLDABLE EQUILIBRIUM LIMITS
% since in this case, u = -1, +1

% Setup PTOS params:
lam = 0.1;
x1r = -0.5*c;
alp = 1;


umax = 1 - x1r/c; % umax*=umax - x1r/c 
umin = -1 - x1r/c; % umin* = umin - x1r/c
e1 = -inv(A)*B*(umin);
e2 = -inv(A)*B*(umax);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  GENERATE THE SWITCHING CURVE
% now integrate backward from the origin but with assymetric control
% limits.
% to Generate  actual curve
Xf = [0 0]'; 
% p3 = plot(x1r, 0, 'xr');


params.u  = umax;
[t, x_pact]=ode45(@(t,x)f(t,x,params), t, Xf);
params.u   = umin; 
[t, x_nact]=ode45(@(t,x)f(t,x,params), t, Xf);

Scurve = [flipud(x_nact); x_pact];
Scurve = forceMono(Scurve);
% p4     = plot(Scurve(:,1)+x1r, Scurve(:,2), 'b');
grid on, hold on;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% make PTOS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


[k2, ptosCurve] = ptosDampRef(sys, Scurve, umin, umax, lam, e1, e2);

%%% Now, let us plot u_dot upper and lower
% Lower, u = umax

fp = ptosCurve.x2_ptos;
x1s = ptosCurve.x1;
dx = diff(x1s);
fpp = diff(fp)./dx;
fpp(end+1) = fpp(end);
udot_max = k2*fpp.*fp + k2*w^2*x1s + fpp*umax - k2*bo*umax;
udot_min = k2*fpp.*fp + k2*w^2*x1s + fpp*umin - k2*bo*umin;

% get crossing points of udot min and max
k1 = crossing(udot_max);
k2 = crossing(udot_min);
%



[AX, H1, H2] = plotyy([x1s, x1s, x1s, Scurve(:,1)],...
                [fp, ptosCurve.x2_upper, ptosCurve.x2_lower Scurve(:,2)],...
                [x1s, x1s], [udot_max, udot_min]);
ylm          = ylim;
hold(AX(1), 'on')
hold(AX(2), 'on')

%
%plot c and xell
kk1 = find(x1s < alp*c*umin, 1, 'last')
kk2 = find(x1s > alp*c*umax, 1, 'first')
% Plot shading
ps2 = plotshaded(x1s(kk1:kk2)', [ptosCurve.x2_upper(kk1:kk2)'; ptosCurve.x2_lower(kk1:kk2)'], 'k', .45);%shading in region B
ps1 = plotshaded(x1s(1:kk1)', [ptosCurve.x2_upper(1:kk1)'; ptosCurve.x2_lower(1:kk1)'], 'k', .10); %shading in T^+
ps3 = plotshaded(x1s(kk2:end)', [ptosCurve.x2_upper(kk2:end)'; ptosCurve.x2_lower(kk2:end)'], 'k', .2);% shading in T^-


% Replot ptos curves
H3 = plot([x1s, x1s, x1s],...
                [fp, ptosCurve.x2_upper, ptosCurve.x2_lower]);
H3(1).Color = CS.fpC;
H3(2).Color = CS.fpupperC;
H3(3).Color = CS.fplowerC;
Hc    = plot([c*umin, c*umin], [ylm], '--b', [c*umax, c*umax], [ylm], '--b');
Hxell = plot([ptosCurve.xell_min, ptosCurve.xell_min], [ylm], '--g', [ptosCurve.xell_plus, ptosCurve.xell_plus], [ylm], '--g');

axes(AX(2))
Hexes = plot(x1s(k1), udot_max(k1), 'xk', x1s(k2), udot_min(k2), 'xk');
axes(AX(1));
%





%%%%%%%%%%%%% 
% set colors

% lower things
H2(1).Color     = CS.fplowerC;
H2(1).LineStyle = '--';
H1(3).Color     = CS.fplowerC;
%
% upper things
H2(2).Color     = CS.fpupperC; % udot upper
H2(2).LineStyle = '--';

H1(2).Color     = CS.fpupperC; %fpupper
H1(2).LineStyle = '-';

% fptos
H1(1).Color     = CS.fpC; 
H1(1).LineStyle = CS.fpS;

%fto
H1(4).Color    = 'k';
H1(4).LineStyle = '--';


% xell and c
Hxell(1).Color     = CS.xellC;
Hxell(1).LineStyle = CS.xellS;
Hxell(2).Color     = CS.xellC;
Hxell(2).LineStyle = CS.xellS;
%
Hc(1).Color     = CS.cC;
Hc(1).LineStyle = CS.cS;
Hc(2).Color     = CS.cC;
Hc(2).LineStyle = CS.cS;
Hc(1).LineWidth = 1.3;
Hc(2).LineWidth = 1.3;
%%%%%%%%%%%%%%%%%
% Create axis labels
xlabel('$x_{1e}$ (error coordinates) ', 'interpreter', 'latex', 'FontSize', 16);
set(get(AX(1), 'YLabel'), 'String', '$x_{2e}$', 'interpreter', 'latex', 'FontSize', 16)
set(get(AX(2), 'YLabel'), 'String', '$\dot u(t)$','interpreter', 'latex', 'FontSize', 16, 'units', 'normalized')
h2 = get(AX(2), 'YLabel');
h2.Position = [1.04, .5 0];

h1 = get(AX(1), 'YLabel');
h1.Position = [-1.5, -5000 0];




%%%%%%%%%%%%%
% set xlimits on BOTH sets of axes
axes(AX(1));
xlim([1.05*2*umin*c, 1.01*2*umax*c]);



axes(AX(2)); grid on; hold on;
xlim([1.05*2*umin*c, 1.01*2*umax*c]);

%%%%%%%%%%%%%%%%%%5
% write text
h_uminc   = text(.135,.04, '$c\cdot \bar u^-$', 'HorizontalAlignment','Right','interpreter', 'latex', 'FontSize', 12, 'Units', 'Normalized');
h_umaxc   = text(.72, .04, '$c \cdot \bar u^+$', 'HorizontalAlignment','Right','interpreter', 'latex', 'FontSize', 12, 'Units', 'Normalized');

h_xlmin   = text(.25, .04, '$-x_{\ell}^-$', 'HorizontalAlignment','Right','interpreter', 'latex', 'FontSize', 12, 'Units', 'Normalized');
h_xlplus  = text(.34, .04, '$x_{\ell}^+$', 'HorizontalAlignment','Right','interpreter', 'latex', 'FontSize', 12, 'Units', 'Normalized');


h_um  = text(.58, .85, '$\mathcal{U}^-$', 'HorizontalAlignment','Right','interpreter', 'latex', 'FontSize', 12, 'Units', 'Normalized');
h_up  = text(.5, .05, '$\mathcal{U}^+$', 'HorizontalAlignment','Right','interpreter', 'latex', 'FontSize', 12, 'Units', 'Normalized');

% h_um  = text(.58, .95, s, 'HorizontalAlignment','Right','interpreter', 'latex', 'FontSize', 12, 'Units', 'Normalized');

h_Bp  = text(.54, .34, '$\mathcal{B}$', 'HorizontalAlignment','Right','interpreter', 'latex', 'FontSize', 12, 'Units', 'Normalized');
h_Bm  = text(.19, .83, '$\mathcal{B}$', 'HorizontalAlignment','Right','interpreter', 'latex', 'FontSize', 12, 'Units', 'Normalized');
h_Tp  = text(.7, .34, '$\mathcal{T}$', 'HorizontalAlignment','Right','interpreter', 'latex', 'FontSize', 12, 'Units', 'Normalized');
h_Tm  = text(.1, .83, '$\mathcal{T}$', 'HorizontalAlignment','Right','interpreter', 'latex', 'FontSize', 12, 'Units', 'Normalized');


% Title String
s = sprintf('$x_{1r} = %.1fc$, $\\lambda = %.1f$', x1r/c, lam);
title(s, 'interpreter', 'latex')

%%%%%%%%%%%%%%%
% create legend
leg = legend([H1(4), H1(1), H1(2), H2(1), H1(3), H2(2)], {'$f_{to,ref}$', '$f_{p,ref}$','$f_{p,ref}^{upper}$', '$\dot u$ lower $(<0)$',...
              '$f_{p,ref}^{lower}$', '$\dot u$ upper $(>0)$'}, 'interpreter', 'latex');
leg.Position = [0.6365 0.6951 0.2308 0.2279];          

%%
if saveFigs
    hgexport(gcf, fig1_path, ...
        hgexport('factorystyle'), 'Format', 'epsc')
end










