%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Get data from experiment
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  froot = '/home/arnold/matlab/TimeOptimal';
close all;
clear all;
clc;
% matpath = 'C:\Users\arnold\Documents\MATLAB\';


figpath   = 'latex/acc_2016_finalForm/figs';

fig1_name = 'experREF_xelp1alpha1_x1rnp5c_timedata.eps';
fig2_name = 'experREF_xelp1alpha1_x1rnp5c_phaseplane.eps';
fig1_path = fullfile(figpath, fig1_name);
fig2_path = fullfile(figpath, fig2_name);

load('data/ptosParamsRef.mat', 'ptosParams')
load('CS.mat', 'CS');

% Load Experimental Data (generated by labview)
dataname ='x1r_p5c.csv';
datapath  = fullfile('data', dataname);
data1=csvread(datapath, 0);

%
saveFigs = 0; % Flag to save figures to figpath

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Expose data into convinient variables
% N_k = length(y);
N_k = 124;

y  = data1(1:N_k,3);
Ts = ptosParams.Ts;
dt = Ts/10;

vhat = data1(1:N_k,2);
yhat = data1(1:N_k,1);
u = data1(1:N_k,4);



t = Ts*[0:1:N_k-1]';
testOn = 0;
if testOn
    figure(1)
    plot(t, u)
    title('control')

    figure(2)
    subplot(2,1,1), hold on, grid on;
    plot(t, y, 'b', t, yhat, '--k')
    title('y')

    subplot(2,1,2), hold on
    plot(t, vhat, 'b')
end

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% setup simulations for comparison
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% load(fullfile(matpath, 'TimeOptimal/LRC/plants.mat'), 'plants')



% pull out continuous time matrices
A = ptosParams.A;
B = ptosParams.B;
C = ptosParams.C;

% alpha = ptosParams.alpha;
c      = ptosParams.c;
TT     = ptosParams.TT;
w      = ptosParams.w;
bo     = ptosParams.bo;
x1r    = ptosParams.x1r;
k2     = ptosParams.k2;
umax   = ptosParams.umax;
umin   = ptosParams.umin;
lambda = ptosParams.lambda;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%
% plus side
xell_plus = ptosParams.xell_plus;
k1_plus   = ptosParams.k1_plus ;
k2_plus   = ptosParams.k2_plus;
cbar_plus = ptosParams.cbar_plus;

Mplus      = k1_plus/k2_plus;
%%%%%%%%%%%%%%%%%%%%%%%%%%%
% minus side
xell_min  = ptosParams.xell_min;
cbar_min  = ptosParams.cbar_min;
k1_min    = ptosParams.k1_min;
k2_min    = ptosParams.k2_min;
Mmin      = k1_min/k2_min;

x1Tol = ptosParams.x1Tol;
x2Tol = ptosParams.x2Tol;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
L = ptosParams.L;

% pull out discrete time matrices
Az = ptosParams.Az;
Bz = ptosParams.Bz;
Cz = ptosParams.Cz;



L = ptosParams.L;


% kxell = find(abs(y)<xell);
% kxell = kxell(1);

% set initial condition
X0   = [y(1), vhat(1)]';
trun = N_k*Ts;
% run the simulation
TDelay = 4*Ts;
Ndelay = 0;
% sim('LRC_linear_discreteObs');
%
sim('ptos_discreteObsRef');
sim('TOPT_REF');
%
% close all, clc
x1Tol = 0.005*c;
x2Tol = .005*(bo/w);
if testOn
    figure(1), hold on;
    plot(uptos.Time, uptos.Data)
    plot(uto.Time, uto.Data, '--g')
    ylim([-1.3, 1.3]);
    grid on;

    figure(2)
    subplot(2,1,1)
    hold on;
    plot(xhat.Time, xhat.Data(:,1), 'm')
    plot(xplant_to.Time, xplant_to.Data(:,1), '--g')
    
    subplot(2,1,2)
    hold on;
    plot(xhat.Time, xhat.Data(:,2), 'm');
    plot(xplant_to.Time, xplant_to.Data(:,2), '--g')
    
    
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Plot Data
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%setup figure
F1 = figure(1);
% F1.Position = [2087 276 548 498];
F1.Position = [-704 441 640 525];
% [-704 441 499 525];
lw = 1.3; % standard linewidth

% plot x1 = y
ye = y-x1r;

alpha = 1;

tend = 1e-3;


% PLOT X1 TIME SERIES
subplot(3,1,1), hold on, grid on
p_x1sim = plot(xplant.Time*1000, xplant.Data(:,1), 'LineWidth', lw);
p_x1exp = plot(t*1000, y, 'LineWidth', lw );
p_x1to  = plot(xplant_to.Time*1000, xplant_to.Data(:,1));

leg1 = legend([p_x1exp, p_x1sim, p_x1to], {'$x_1$ (experiment, PTOS$\omega$)', ...
       '$x_1$ (simulation, PTOS$\omega$)', '$x_1$ (simulation, time-optimal)'},...
        'interpreter', 'latex', 'FontSize', 12);
leg1.Position = [0.4769 0.7982 0.4249 0.1231];
% xlabel('t [s]');
ylabel('$x_1$', 'interpreter', 'latex', 'FontSize', 16);
s = sprintf('$x_{1r} = %.2fc$, $\\lambda = %.2f$', x1r/c, lambda);
title(s, 'interpreter', 'latex', 'FontSize', 16);
xlim([0, tend*1000])
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%5
% plot x2 = v
subplot(3,1,2), hold on, grid on;
p_x2sim = plot(xplant.Time*1000, xplant.Data(:,2), 'LineWidth', lw);
p_x2exp = plot(t*1000, vhat, 'LineWidth', lw);
p_x2to  = plot(xplant_to.Time*1000, xplant_to.Data(:,2));
%
% plot(t(kxell), vhat(kxell), 'xk', 'MarkerSize', 5, 'LineWidth', 2)
leg2 = legend([p_x2exp, p_x2sim, p_x2to], {'$\hat x_2$ (experiment, PTOS$\omega$)',...
    '$x_2$ (simulation, PTOS$\omega$)', '$x_2$ (simulation, time-optimal)'},...
    'interpreter', 'latex', 'FontSize',12);
leg2.Position = [0.4810 0.4139 0.4210 0.1224];
% xlabel('t [s]')
ylabel('$x_2, \hat x_2$', 'interpreter', 'latex', 'FontSize', 16);
xlim([0, tend*1000]);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% plot control history
subplot(3,1,3), hold on, grid on;
p_usim = plot(uptos.Time*1000, uptos.Data, 'LineWidth', lw);
p_uexp = plot(t*1000, u, 'LineWidth', lw );
p_uto  = plot(uto.Time*1000, uto.Data);
% plot(t(kxell), u(kxell), 'xk', 'MarkerSize', 5, 'LineWidth', 2)
leg3 = legend([p_uexp, p_usim, p_uto], {'u (experiment, PTOS$\omega$)', 'u (simulation, PTOS$\omega$)',...
    'u (simulation, time-optimal)'}, 'interpreter', 'latex', 'FontSize', 12);
leg3.Position = [0.4901 0.2002 0.4110 0.1224];

xlim([0, tend*1000])
xlabel('t [ms]')
ylabel('$u(t)$', 'interpreter', 'latex', 'FontSize', 16)


%%%%%%%%%%%%%%%%%
% colors and linestyles
%x1 
p_x1exp.Color     = CS.expC;
p_x1exp.LineStyle = CS.expS;
p_x1sim.Color     = CS.ptosSimC;
p_x1sim.LineStyle = CS.ptosSimS;
p_x1to.Color      = CS.toSimC;
p_x1to.LineStyle  = CS.toSimS;

%
%x2
p_x2exp.Color     = CS.expC;
p_x2exp.LineStyle = CS.expS;
p_x2sim.Color     = CS.ptosSimC;
p_x2sim.LineStyle = CS.ptosSimS;
p_x2to.Color      = CS.toSimC;
p_x2to.LineStyle  = CS.toSimS;

% u
p_uexp.Color     = CS.expC;
p_uexp.LineStyle = CS.expS;
p_usim.Color     = CS.ptosSimC;
p_usim.LineStyle = CS.ptosSimS;
p_uto.Color      = CS.toSimC;
p_uto.LineStyle  = CS.toSimS;


if saveFigs
    hgexport(gcf, fig1_path, ...
        hgexport('factorystyle'), 'Format', 'epsc')
end



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Construct PTOS plot, and phase plane trajectory
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% close all
% clf
x1 = -2*cbar_min:.001:2*cbar_plus;
for i =1:length(x1)
    fp(i) = fptosref(x1(i), ptosParams);
end
%
if 0
    figure(3)
    plot(x1, fp);, grid on, hold on;
    plot(x1, fp-umax/k2, x1, fp+abs(umin)/k2);
    plot(xhat.Data(:,1)-x1r, xhat.Data(:,2));
    plot(y-x1r, vhat);
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

x1_ptos = x1;
fptos   = fp;

% setup figure stuff
lw = 1.5;
F2=figure(2);
F2.Position = [-1570 325 693 498];
% [-713 -119 548 498];
hold on, grid on


%%%%%%%%%%%%%%%%%%%%%%%%
% plot ptos curve and boundaries
p1       = plot(x1_ptos, fptos, 'LineWidth', lw); 
p2       = plot(x1_ptos, fptos+abs(umin)/k2, 'LineWidth', lw); 
p3       = plot(x1_ptos, fptos-umax/k2, 'LineWidth', lw);

%%%%%%%%%%%%%%%%%%%%%%%%
% plot trajectories
pp1= plot(xplant.Data(:,1)-x1r, xplant.Data(:,2),  'LineWidth', lw);
pp2 = plot(y-x1r, vhat, 'LineWidth', lw);
ylm = ylim;

%%%%%%%%%%%%%%%%%%%%%%%
% plot xell and c vertical lines
p_xell = plot([xell_plus, xell_plus], ylm, 'g', [-xell_min, -xell_min], ylm, 'g', 'LineWidth', lw);
p_c    = plot([c*umax, c*umax], ylm, 'b', [c*umin, c*umin], ylm, 'b', 'LineWidth', lw);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Set colors and linestyles
% trajectories
pp1.Color     = CS.ptosSimC;
pp2.Color     = CS.expC;
pp2.LineStyle = CS.expS;

%%%%%%%%%%%%%%
% fptos and boundares
p1.Color = CS.fpC;
p2.Color = CS.fpupperC;
p3.Color = CS.fplowerC;

%%%%%%%%%%%%%%%
% xell and c
p_xell(1).Color     = CS.xellC;
p_xell(2).Color     = CS.xellC;
p_c(1).Color        = CS.c;
p_c(2).Color        = CS.c;


%     s = sprintf('xell = %fc, alpha = %f', xell/c, alpha)
%     s = sprintf('$x_{\\ell}^+ = %.2fc$, $\\alpha = %.2f$', xell_plus/c, alpha)

%%%%%%%%%%%%%%%%%%%%%%%%%
% create LABELS and TITLE
title(s, 'interpreter', 'latex', 'FontSize', 16)
xlim([1.5*umin*c 1.2*c*umax]);
xlabel('$x_{1e}$ (error coordinates)', 'interpreter', 'latex', 'FontSize', 16);
ylabel('$x_{2e}$', 'interpreter', 'latex', 'FontSize', 16, 'rot', 0);

%%%%%%%%%%%%%%%%%%%%%%%%%
% write text

h_plus=text(0.22, 0.2, '$\mathcal{U}^+$', 'HorizontalAlignment','Right','interpreter', 'latex', 'FontSize', 16, 'Units', 'Normalized');
h_min=text(.80,.70, '$\mathcal{U}^-$', 'HorizontalAlignment','Right','interpreter', 'latex', 'FontSize', 16, 'Units', 'Normalized');
h_Bm=text(.24,.8, '$\mathcal{B}$', 'HorizontalAlignment','Right','interpreter', 'latex', 'FontSize', 16, 'Units', 'Normalized');
h_Bp=text(.54,.3, '$\mathcal{B}$', 'HorizontalAlignment','Right','interpreter', 'latex', 'FontSize', 16, 'Units', 'Normalized');

h_Tm=text(.04,.84, '$\mathcal{T}$', 'HorizontalAlignment','Right','interpreter', 'latex', 'FontSize', 16, 'Units', 'Normalized');
h_Tp=text(.94,.13, '$\mathcal{T}$', 'HorizontalAlignment','Right','interpreter', 'latex', 'FontSize', 16, 'Units', 'Normalized');

h_xln=text(.275,.04, '$-x^-_{\ell}$', 'HorizontalAlignment','Right','interpreter', 'latex', 'FontSize', 12, 'Units', 'Normalized');
h_xlp=text(.4,.04, '$x^+_{\ell}$', 'HorizontalAlignment','Right','interpreter', 'latex', 'FontSize', 12, 'Units', 'Normalized');
h_xln=text(.09,.04, '$c \bar u^-$', 'HorizontalAlignment','Right','interpreter', 'latex', 'FontSize', 12, 'Units', 'Normalized');
h_xlp=text(.95,.04, '$c \bar u^+$', 'HorizontalAlignment','Right','interpreter', 'latex', 'FontSize', 12, 'Units', 'Normalized');

%%%%%%%%%%%%%%%%%%%%%%%%
% create legend
leg2 = legend([pp1, pp2, p1, p2, p3 ], {'simulation', 'experiment', '$f_{ptos}(x_{1e})$', '$\mathcal{T}$ upper boundary', '$\mathcal{T}$ lower boundary' }, 'interpreter', 'latex');
leg2.Position = [0.5624 0.7153 0.3406 0.2088];
leg2.FontSize = 12;





if saveFigs
    hgexport(gcf, fig2_path, ...
        hgexport('factorystyle'), 'Format', 'epsc')
end


